# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging
import os

from txwebbackendbase.requests import dejsonify
from txwebbackendbase.requests import jsonify
from txwebbackendbase.singleton import singleton

from klein import Klein
from twisted.internet.defer import inlineCallbacks
from twisted.internet.defer import returnValue

from dopplerr.downloader import Downloader
from dopplerr.status import DopplerrStatus

log = logging.getLogger(__name__)


@singleton
class Routes(object):
    app = Klein()
    args = None

    def listen(self):
        self.app.run(host='0.0.0.0', port=int(DopplerrStatus().port))

    @app.route("/")
    def root(self, _request):
        return ("Status page not implemented Yet. <br>"
                "Use /health for health information <br><br>")

    @app.route("/notify", methods=['POST'])
    @inlineCallbacks
    def notify(self, request):
        if DopplerrStatus().appdir:
            os.chdir(DopplerrStatus().appdir)
        content = dejsonify(request)
        logging.debug("Notify request: %r", content)
        res = yield Downloader().process_notify_request(content)
        returnValue(jsonify(request, res))

    @app.route("/notify", methods=['GET'])
    def notify_not_allowed(self, request):
        request.setResponseCode(405)
        return "Method GET not allowed. Use POST with a JSON body with the right format"

    @app.route("/health")
    def health(self, request):
        res_health = {
            "healthy": DopplerrStatus().healthy,
            "languages": DopplerrStatus().languages,
            "mapping": DopplerrStatus().path_mapping,
        }
        return jsonify(request, res_health)

    @app.route("/fullscan")
    @inlineCallbacks
    def fullscan(self, request):
        content = dejsonify(request)
        logging.debug("Fullscan request: %r", content)
        res = yield Downloader().process_fullscan(content)
        returnValue(jsonify(request, res))



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from txwebbackendbase.singleton import singleton


@singleton
class DopplerrStatus(object):
    def __init__(self):
        self.__healthy = False
        self.__basedir = None
        self.__path_mapping = None
        self.__appdir = None
        self.__configdir = None
        self.__basedir = None
        self.__port = 0
        self.__languages = []

    @property
    def healthy(self):
        return self.__healthy

    @healthy.setter
    def healthy(self, healthy):
        self.__healthy = healthy

    @property
    def basedir(self):
        return self.__basedir

    @basedir.setter
    def basedir(self, basedir):
        self.__basedir = basedir

    @property
    def path_mapping(self):
        return self.__path_mapping

    @path_mapping.setter
    def path_mapping(self, mapping):
        self.__path_mapping = mapping

    @property
    def appdir(self):
        return self.__appdir

    @appdir.setter
    def appdir(self, appdir):
        self.__appdir = appdir

    @property
    def configdir(self):
        return self.__configdir

    @configdir.setter
    def configdir(self, configdir):
        self.__configdir = configdir

    @property
    def port(self):
        return self.__port

    @port.setter
    def port(self, port):
        self.__port = port

    @property
    def languages(self):
        return self.__languages

    @languages.setter
    def languages(self, languages):
        self.__languages = languages



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import argparse
import logging
import os
import sys

from txwebbackendbase.logging import setupLogger

from dopplerr.downloader import Downloader
from dopplerr.routes import Routes
from dopplerr.status import DopplerrStatus

log = logging.getLogger(__name__)
ALLOWED_LANGUAGES = ["fra", "eng", "ger"]


def print_list(aList):
    return ",".join(aList)


def list_of_languages(langList):
    langs = [s.lower() for s in langList.split(',')]
    failed = False
    for l in langs:
        if l not in ALLOWED_LANGUAGES:
            failed = True
            logging.fatal("Invalid language: %r")
    if failed:
        logging.fatal("List of allowed languages: %s", print_list(ALLOWED_LANGUAGES))
        raise argparse.ArgumentTypeError("Invalid language")
    return langs


def inject_env_variables(argv):
    languages = os.environ.get("SUBDLSRC_LANGUAGES")
    if languages:
        argv.append(["--languages", languages])
    basedir = os.environ.get("SUBDLSRC_BASEDIR")
    if basedir:
        argv.extend(["--basedir", basedir])
    verbose = os.environ.get("SUBDLSRC_VERBOSE")
    if str(verbose).strip() == "1":
        argv.extend(["--verbose"])
    logfile = os.environ.get("SUBDLSRC_LOGFILE")
    if logfile:
        argv.extend(["--logfile", logfile])


def main():
    argv = sys.argv[1:]
    inject_env_variables(argv)
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', '--port', action='store', dest='port', help='The port to listen on')
    parser.add_argument(
        '-b', '--basedir', action='store', dest='basedir', help='Base directory', default="")
    parser.add_argument(
        '-c', '--configdir', action='store', dest='configdir', help='Config directory')
    parser.add_argument(
        '-a', '--appdir', action='store', dest='appdir', help='App directory', default="")
    parser.add_argument(
        '-n',
        '--no-color',
        action='store_true',
        dest='no_color',
        help='Disable color in logs',
        default=False)
    parser.add_argument(
        '-v',
        '--verbose',
        action='store_true',
        dest='verbose',
        help='Verbose output',
        default=False)
    parser.add_argument(
        '--mapping',
        action="store",
        dest="path_mapping",
        nargs="*",
        help=("Map root folder of tv/anime/movie to another name.\n"
              "Ex: series are mounted on a docker image as /tv but \n"
              "on the other system it is under /video/Series. In this \n"
              "case use '--basedir /video --mapping tv=Series movies=Movies'"))
    parser.add_argument(
        "-l",
        "--languages",
        action="store",
        dest="languages",
        help="Wanted languages (comma separated list)",
        type=list_of_languages,
        required=True)
    parser.add_argument(
        "--logfile",
        action="store",
        dest="logfile",
        help="Output log to file",
    )

    args = parser.parse_args(args=argv)
    setupLogger(
        level=logging.DEBUG if args.verbose else logging.WARNING,
        no_color=args.no_color,
        logfile=args.logfile)
    log.info("Initializing Subtitle Downloader Service")

    if args.port is None:
        print("Missing required argument: -p/--port")
        sys.exit(1)
    log.debug("Starting listening on port %s", args.port)
    log.debug("Application directory: %s", args.appdir)
    log.debug("Media base directory: %s", args.basedir)
    log.debug("Config directory: %s", args.configdir)
    log.debug("Wanted languages: %s", args.languages)
    if not args.languages or any(not x for x in args.languages):
        raise Exception("Bad languages: {!r}".format(args.languages))
    if args.path_mapping:
        log.debug("Path Mapping: %r", args.path_mapping)
    Downloader.initialize_subliminal()
    DopplerrStatus().healthy = True
    DopplerrStatus().appdir = args.appdir
    DopplerrStatus().port = args.port
    DopplerrStatus().path_mapping = args.path_mapping
    DopplerrStatus().configdir = args.configdir
    DopplerrStatus().languages = args.languages
    Routes().listen()


if __name__ == '__main__':
    main()
